<?php
// Holger Irmler
// 07-2019
// https://cmsimpleforum.com/viewtopic.php?f=16&t=15164&start=60#p72167

class BwMenuLi extends XH_Li {

    /**
     * Renders the ul start tags.
     *
     * @param int $i The index of the current item.
     *
     * @return string (X)HTML.
     *
     * @global array The menu levels of the pages.
     *
     * @access protected
     */
    function renderULStartTags($i) {
        global $l;

        $lines = array();
        for ($k = $this->getMenuLevel($i - 1); $k < $this->getMenuLevel($i);
                    $k++) {
            if ($k === 0) {
                //optional: zus. Klasse bwMenu und ID menu-main-navigation
                $lines[] = "\n" . '<ul class="bwMenu ' . $this->st
                        . ($k + 1) . '" id="menu-main-navigation">' . "\n";
            } else {
                $lines[] = "\n" . '<ul class="'
                        . $this->st . ($k + 1) . '">' . "\n";
            }
        }
        return implode('<li>' . "\n", $lines);
    }

    /**
     * Returns the class name of the current item.
     *
     * @param int $i The index of the current item.
     *
     * @return string
     *
     * @global array  The configuration of the core.
     *
     * @access protected
     */
    function getClassName($i) {
        $className = '';
        if ($this->isSelected($i)) {
            $className .= 's';
        }
        $className .= 'doc';
        if ($this->hasChildren($i)) {
            $className .= 's';
        }
        return $className;
    }

    /**
     * Renders a menu item.
     *
     * @param int $i The index of the current item.
     *
     * @return string (X)HTML.
     *
     * @global array  The headings of the pages.
     *
     * @access protected
     */
    function renderMenuItem($i) {
        global $c, $edit, $l, $h;

        if ($this->hasChildren($i) && $l[$i] === '1') {
            // kein Seitenlink wenn
            // - im Admin-Mode, wenn Vorschau aktiv
            // - Kind-Seite vorhanden ist
            // - aktueller Seiten-Level '1' ist
            if (XH_ADM && $edit) {
                //im Edit-Mode Seiten-Link erzeugen
                $menuEntry = $this->renderAnchorStartTag($i)
                        . $h[$this->ta[$i]] . '</a>';
            } else {
                $menuEntry = $h[$this->ta[$i]];
            }
            $t = '
<div class="bwCategory">' . $menuEntry . '</div>'
                    . '
<div class="bwDescription">'
                    . preg_replace('/<!--XH_ml[1-9]:.*?-->/is', '', trim($c[$i]))
                    . '</div>';
        } else {
            //Alle Eintraege klickbar lassen
            $t = $this->renderAnchorStartTag($i) . $h[$this->ta[$i]] . '</a>';
        }
        return $t;
    }

    /**
     * Renders an anchor start tag.
     *
     * @param int $i The index of the current item.
     *
     * @return string HTML
     */
    function renderAnchorStartTag($i) {
        $x = !$this->tf ? '" class="bwCurrent' : ''; //Klasse "bwCurrent" dazu
        $x .= $this->shallOpenInNewWindow($i) ? '" target="_blank' : '';
        return a($this->ta[$i], $x);
    }

}

class BwSitemapLi extends XH_Li {

    /**
     * Renders a menu item.
     *
     * @param int $i The index of the current item.
     *
     * @return string (X)HTML.
     *
     * @global array  The headings of the pages.
     *
     * @access protected
     */
    function renderMenuItem($i) {
        global $c, $edit, $l, $h;

        if ($this->hasChildren($i) && $l[$i] === '1') {
            // kein Seitenlink wenn
            // - Kind-Seite vorhanden ist
            // - aktueller Seiten-Level '1' ist
            $t = '<div class="bwSitemapCategory">' . $h[$this->ta[$i]] . '</div>';
        } else {
            //Alle Eintraege klickbar lassen
            $t = $this->renderAnchorStartTag($i) . $h[$this->ta[$i]] . '</a>';
        }
        return $t;
    }

}

// Eigentlich eine komplette Kopie von XH_Search.
// Leider sind die wesentlichen Methoden im Core als "private"
// deklariert. Daher muss, auch für eine so kleine Anpassung,
// eine Komplette Kopie der Klasse vorgehalten werden...
// ... Nicht optimal :( .
class BwSearch {

    /**
     * The search String.
     *
     * @var string
     */
    private $searchString;

    /**
     * The search words.
     *
     * @var array
     */
    private $words;

    /**
     * Constructs an instance.
     *
     * @param string $searchString String The search string.
     */
    public function __construct($searchString) {
        $this->searchString = $searchString;
    }

    /**
     * Returns the array of search words.
     *
     * @return array
     */
    private function getWords() {
        if (!isset($this->words)) {
            $words = explode(' ', $this->searchString);
            $this->words = array();
            foreach ($words as $word) {
                $word = trim($word);
                if ($word != '') {
                    if (class_exists('\Normalizer', false) && method_exists('\Normalizer',
                                    'normalize')
                    ) {
                        $word = \Normalizer::normalize($word);
                    }
                    $this->words[] = $word;
                }
            }
        }
        return $this->words;
    }

    /**
     * Returns an array of page indexes
     * where all words of the search string are contained.
     *
     * @return array
     *
     * @global array The content of the pages.
     * @global array The configuration of the core.
     */
    public function search() {
        global $c, $cf, $l;

        $result = array();
        $words = $this->getWords();
        if (empty($words)) {
            return $result;
        }
        foreach ($c as $i => $content) {
            //Einzige Änderung: Top-Level-Seiten ignorieren
            if ($l[$i] > 1 && !hide($i) || $cf['show_hidden']['pages_search'] == 'true') {
                $found = true;
                $content = $this->prepareContent($content, $i);
                foreach ($words as $word) {
                    if (utf8_stripos($content, $word) === false) {
                        $found = false;
                        break;
                    }
                }
                if ($found) {
                    $result[] = $i;
                }
            }
        }
        return $result;
    }

    /**
     * Prepares content to be searched.
     *
     * @param string $content   A content.
     * @param string $pageIndex A page index.
     *
     * @return string
     */
    private function prepareContent($content, $pageIndex) {
        global $s;

        $vars = array('s', 'o', 'hjs', 'bjs', 'e', 'onload');
        foreach ($vars as $var) {
            $old[$var] = $GLOBALS[$var];
        }
        $s = $pageIndex;
        $content = strip_tags(evaluate_plugincall($content));
        foreach ($vars as $var) {
            $GLOBALS[$var] = $old[$var];
        }
        if (method_exists('\Normalizer', 'normalize')) {
            $content = \Normalizer::normalize($content);
        }
        return html_entity_decode($content, ENT_QUOTES, 'UTF-8');
    }

    /**
     * Returns a message how often the search string was found.
     *
     * @param int $count How often the search string was found.
     *
     * @return string HTML
     *
     * @global array The localization of the core.
     */
    private function foundMessage($count) {
        global $tx;

        if ($count == 0) {
            $key = 'notfound';
        } elseif ($count == 1) {
            $key = 'found_1';
        } elseif (2 <= $count && $count <= 4) {
            $key = 'found_2-4';
        } else {
            $key = 'found_5';
        }
        $message = sprintf($tx['search'][$key], $this->searchString, $count);
        $message = XH_hsc($message);
        $message = '<p>' . $message . '</p>';
        return $message;
    }

    /**
     * Returns the search results view.
     *
     * @return string HTML
     *
     * @global array  The headings of the pages.
     * @global array  The URLs of the pages.
     * @global string The script name.
     * @global array  The configuration of the core.
     * @global array  The localization of the core.
     * @global object The page data router.
     */
    public function render() {
        global $h, $u, $sn, $cf, $tx, $pd_router;

        $cf['meta']['robots'] = 'noindex, nofollow';
        $o = '<h1>' . $tx['search']['result'] . '</h1>';
        $words = $this->getWords();
        $pages = $this->search();
        $count = count($pages);
        $o .= $this->foundMessage($count) . PHP_EOL;
        if ($count > 0) {
            $o .= '<ul>' . PHP_EOL;
            $words = implode(' ', $words);
            foreach ($pages as $i) {
                $pageData = $pd_router->find_page($i);
                $site = isset($pageData['title']) ? $pageData['title'] : '';
                $title = XH_title($site, $h[$i]);
                $url = $sn . '?' . $u[$i] . '&amp;search=' . urlencode($words);
                $o .= '    <li><a href="' . $url . '">' . $title . '</a>';
                $description = isset($pageData['description']) ? $pageData['description']
                            : '';
                if ($description != '') {
                    $o .= '<div>' . XH_hsc($description) . '</div>';
                }
                $o .= '</li>' . PHP_EOL;
            }
            $o .= '</ul>' . PHP_EOL;
        }
        return $o;
    }

}

function bwFindLeaf(XH\Pages $pages, $index) {
    if ($index >= 0) {
        $children = $pages->children($index);
        if (empty($children)) {
            return $index;
        } else {
            return bwFindLeaf($pages, $children[0]);
        }
    }
}

function bwRedirect() {
    global $l, $o, $s;

    // Umleiten nur
    // - wenn Seiten-Level === 1 ist
    // - Seite Kindseiten hat
    // - CMS nicht im Admin-Modus ist
    if ($s > -1 && $l[$s] === '1') {
        $pages = new XH\Pages;
        $newIndex = bwFindLeaf($pages, $s);
        if ($newIndex !== $s) {
            //Seite hat Unterseiten
            if ($s >= 0 && !XH_ADM) {
                header('Location: ' . CMSIMPLE_URL . '?'
                        . $pages->url($newIndex), true, 302);
                exit;
            } else {
                if ($s >= 0) {
                    $msg = 'Note: The names of level 1 pages are only used as category headings. The contents of these pages become category descriptions.';
                    //Hinweis fuer User über Editor ausgeben
                    $o = XH_message('warning', $msg) . $o;
                }
            }
        }
    }
}

function bwRenderSitemap() {
    global $cl, $tx, $o, $pd_router, $title;

    $title = $tx['title']['sitemap'];
    $pages = array();
    $o = '<h1>' . $title . '</h1>' . "\n";
    for ($i = 0; $i < $cl; $i++) {
        $pageData = $pd_router->find_page($i);
        if (XH_ADM || (!hide($i) && $pageData['linked_to_menu'])) {
            $pages[] = $i;
        }
    }
    $t = new BwSitemapLi();
    $o .= $t->render($pages, 'sitemaplevel');
}

function bwHandleSearch() {
    global $pth, $tx, $title, $o, $search;

    if (file_exists($pth['file']['search'])) {
        // For compatibility with modified search functions and search plugins.
        include $pth['file']['search'];
    } else {
        $title = $tx['title']['search'];
        $t = new BwSearch(stsl($search));
        $o = $t->render();
    }
}

/* * ************************************************************************ */

//Funktion zur Ausgabe des Menues im Template
//Beispiel: echo bwMenu($hc,1);
function bwMenu($ta, $st) {
    $li = new BwMenuLi();
    return $li->render($ta, $st);
}

//Weiterleitungen prüfen / ausfuehren
bwRedirect();

//handle Sitemap
if ($f == 'sitemap') {
    bwRenderSitemap();
    $f = 'bw_sitemap';
}

//handle Search
if ($f == 'search') {
    bwHandleSearch();
    $f = 'bw_search';
}